"""WireGuard 管理器核心模块"""

import re
import subprocess
from datetime import datetime
from pathlib import Path
from typing import Optional

from .config import CONFIG_DIR, DB_FILE, EXPORT_DIR, DEFAULT_DNS, DEFAULT_MTU, generate_random_port
from .crypto import generate_keypair, generate_preshared_key, generate_public_key
from .database import Database
from .models import Peer, ServerConfig
from .ssh import SSHClient, SSHConfig, RemoteWireGuard


class WireGuardManager:
    """WireGuard 管理器"""

    def __init__(self, server_id: Optional[int] = None):
        self.config_dir = CONFIG_DIR
        self.export_dir = EXPORT_DIR
        self._ensure_dirs()
        self.db = Database(DB_FILE)
        self._server_id = server_id
        self._load_data()
        self._ssh_client: Optional[SSHClient] = None
        self._remote_wg: Optional[RemoteWireGuard] = None

    def _ensure_dirs(self):
        """确保目录存在"""
        self.config_dir.mkdir(parents=True, exist_ok=True)
        self.export_dir.mkdir(parents=True, exist_ok=True)

    def _load_data(self):
        """加载配置数据"""
        servers = self.db.get_servers()
        if self._server_id:
            self.server = self.db.get_server(self._server_id) or ServerConfig()
        elif servers:
            # 默认使用第一个服务端
            self.server = servers[0]
            self._server_id = self.server.id
        else:
            self.server = ServerConfig()
            self._server_id = None

        if self._server_id:
            self.peers = self.db.get_peers(self._server_id)
        else:
            self.peers = []

    def _refresh_peers(self):
        """刷新客户端列表"""
        if self._server_id:
            self.peers = self.db.get_peers(self._server_id)
        else:
            self.peers = []

    @property
    def server_id(self) -> Optional[int]:
        """当前服务端 ID"""
        return self._server_id

    # ========== 多服务端管理 ==========

    def get_servers(self) -> list[ServerConfig]:
        """获取所有服务端"""
        return self.db.get_servers()

    def switch_server(self, server_id: int) -> bool:
        """切换到指定服务端"""
        server = self.db.get_server(server_id)
        if server:
            self._server_id = server_id
            self.server = server
            self._refresh_peers()
            self._ssh_client = None
            self._remote_wg = None
            return True
        return False

    def switch_server_by_endpoint(self, endpoint: str, interface: str = None) -> bool:
        """根据 endpoint（和可选的 interface）切换服务端"""
        if interface:
            # 精确匹配 endpoint + interface
            server = self.db.get_server_by_endpoint_and_interface(endpoint, interface)
        else:
            # 只匹配 endpoint，如果有多个则返回第一个
            server = self.db.get_server_by_endpoint(endpoint)
        if server:
            return self.switch_server(server.id)
        return False

    def delete_server(self, server_id: int) -> bool:
        """删除服务端及其所有客户端"""
        if self._server_id == server_id:
            # 如果删除的是当前服务端，切换到其他服务端
            servers = self.db.get_servers()
            other_server = next((s for s in servers if s.id != server_id), None)
            if other_server:
                self.switch_server(other_server.id)
            else:
                self._server_id = None
                self.server = ServerConfig()
                self.peers = []

        return self.db.delete_server(server_id)

    # ========== SSH 相关 ==========

    def setup_ssh(self, host: str, port: int = 22, user: str = "root",
                  key_file: Optional[str] = None) -> tuple[bool, str]:
        """配置 SSH 连接"""
        if not self._server_id:
            return False, "请先初始化或选择服务端"

        config = SSHConfig(host=host, port=port, user=user, key_file=key_file)
        self._ssh_client = SSHClient(config)

        success, msg = self._ssh_client.test_connection()
        if success:
            self.db.save_ssh_config(self._server_id, host, port, user)
            self._remote_wg = RemoteWireGuard(self._ssh_client, self.server.interface)
        return success, msg

    def get_ssh_client(self) -> Optional[SSHClient]:
        """获取 SSH 客户端"""
        if self._ssh_client:
            return self._ssh_client

        if not self._server_id:
            return None

        ssh_config = self.db.get_ssh_config(self._server_id)
        if ssh_config:
            config = SSHConfig(**ssh_config)
            self._ssh_client = SSHClient(config)
            return self._ssh_client
        return None

    def get_remote_wg(self) -> Optional[RemoteWireGuard]:
        """获取远程 WireGuard 管理器"""
        if self._remote_wg:
            return self._remote_wg

        client = self.get_ssh_client()
        if client and self.server.interface:
            self._remote_wg = RemoteWireGuard(client, self.server.interface)
            return self._remote_wg
        return None

    # ========== 服务端管理 ==========

    def init_server(self, endpoint: str, address: str = "10.0.0.1/24",
                    port: int = 51820, interface: str = "wg0") -> ServerConfig:
        """初始化服务端配置"""
        # 检查 endpoint + interface 组合是否已存在
        existing = self.db.get_server_by_endpoint_and_interface(endpoint, interface)
        if existing:
            raise ValueError(f"服务端 '{endpoint}:{interface}' 已存在")

        private_key, public_key = generate_keypair()

        try:
            result = subprocess.run(
                ["ip", "route", "show", "default"],
                capture_output=True, text=True
            )
            default_interface = result.stdout.split()[4] if result.stdout else "eth0"
        except:
            default_interface = "eth0"

        post_up = f"iptables -A FORWARD -i %i -j ACCEPT; iptables -t nat -A POSTROUTING -o {default_interface} -j MASQUERADE"
        post_down = f"iptables -D FORWARD -i %i -j ACCEPT; iptables -t nat -D POSTROUTING -o {default_interface} -j MASQUERADE"

        self.server = ServerConfig(
            id=None,  # 新服务端，让数据库自动分配 ID
            private_key=private_key,
            public_key=public_key,
            address=address,
            listen_port=port,
            interface=interface,
            endpoint=endpoint,
            post_up=post_up,
            post_down=post_down
        )
        self.server = self.db.save_server(self.server)
        self._server_id = self.server.id
        self._refresh_peers()
        return self.server

    def import_server(self, private_key: str, endpoint: str,
                      address: str = "10.0.0.1/24", port: int = 51820,
                      interface: str = "wg0", post_up: str = "",
                      post_down: str = "") -> ServerConfig:
        """导入已有服务端配置（通过私钥）"""
        # 检查 endpoint + interface 组合是否已存在
        existing = self.db.get_server_by_endpoint_and_interface(endpoint, interface)
        if existing:
            # 更新现有服务端
            self.server = existing
            self._server_id = existing.id
        else:
            self.server = ServerConfig(id=None)

        public_key = generate_public_key(private_key)

        self.server.private_key = private_key
        self.server.public_key = public_key
        self.server.address = address
        self.server.listen_port = port
        self.server.interface = interface
        self.server.endpoint = endpoint
        self.server.post_up = post_up
        self.server.post_down = post_down

        self.server = self.db.save_server(self.server)
        self._server_id = self.server.id
        self._refresh_peers()
        return self.server

    def import_server_from_config(self, config_path: str, endpoint: str) -> tuple[ServerConfig, list[dict]]:
        """从现有配置文件导入服务端"""
        config_file = Path(config_path)
        if not config_file.exists():
            raise FileNotFoundError(f"配置文件不存在: {config_path}")

        content = config_file.read_text()
        return self._parse_and_import_config(content, endpoint, config_file.stem)

    def import_server_from_remote(self, endpoint: str) -> tuple[ServerConfig, list[dict]]:
        """从远程服务器导入配置"""
        remote_wg = self.get_remote_wg()
        if not remote_wg:
            raise RuntimeError("SSH 未配置")

        success, content = remote_wg.get_config()
        if not success:
            raise RuntimeError(f"读取远程配置失败: {content}")

        return self._parse_and_import_config(content, endpoint, self.server.interface or "wg0")

    def _parse_and_import_config(self, content: str, endpoint: str,
                                  interface: str) -> tuple[ServerConfig, list[dict]]:
        """解析配置内容并导入"""
        private_key = ""
        address = "10.0.0.1/24"
        port = 51820
        post_up = ""
        post_down = ""

        interface_match = re.search(r'\[Interface\](.*?)(?=\[Peer\]|$)', content, re.DOTALL | re.IGNORECASE)
        if interface_match:
            interface_section = interface_match.group(1)

            pk_match = re.search(r'PrivateKey\s*=\s*(\S+)', interface_section)
            if pk_match:
                private_key = pk_match.group(1)

            addr_match = re.search(r'Address\s*=\s*(\S+)', interface_section)
            if addr_match:
                address = addr_match.group(1)

            port_match = re.search(r'ListenPort\s*=\s*(\d+)', interface_section)
            if port_match:
                port = int(port_match.group(1))

            postup_match = re.search(r'PostUp\s*=\s*(.+)$', interface_section, re.MULTILINE)
            if postup_match:
                post_up = postup_match.group(1).strip()

            postdown_match = re.search(r'PostDown\s*=\s*(.+)$', interface_section, re.MULTILINE)
            if postdown_match:
                post_down = postdown_match.group(1).strip()

        if not private_key:
            raise ValueError("配置文件中未找到 PrivateKey")

        server = self.import_server(
            private_key=private_key,
            endpoint=endpoint,
            address=address,
            port=port,
            interface=interface,
            post_up=post_up,
            post_down=post_down
        )

        existing_peers = []
        peer_matches = re.finditer(r'\[Peer\](.*?)(?=\[Peer\]|$)', content, re.DOTALL | re.IGNORECASE)
        for i, match in enumerate(peer_matches):
            peer_section = match.group(1)

            pub_match = re.search(r'PublicKey\s*=\s*(\S+)', peer_section)
            allowed_match = re.search(r'AllowedIPs\s*=\s*(\S+)', peer_section)
            psk_match = re.search(r'PresharedKey\s*=\s*(\S+)', peer_section)
            comment_match = re.search(r'#\s*(.+)$', peer_section, re.MULTILINE)

            if pub_match:
                name = comment_match.group(1).strip() if comment_match else f"imported_peer_{i+1}"
                existing_peers.append({
                    "name": name,
                    "public_key": pub_match.group(1),
                    "preshared_key": psk_match.group(1) if psk_match else "",
                    "allowed_ips": allowed_match.group(1) if allowed_match else ""
                })

        return server, existing_peers

    # ========== 客户端管理 ==========

    def _get_next_ip(self) -> str:
        """获取下一个可用 IP"""
        base = self.server.address.split("/")[0]
        parts = base.split(".")
        base_prefix = ".".join(parts[:3])

        used_ips = self.db.get_used_ips(self._server_id)

        for i in range(2, 255):
            if i not in used_ips:
                return f"{base_prefix}.{i}/32"

        raise RuntimeError("IP 地址池已满")

    def _get_server_network(self) -> str:
        """获取服务端网段 (如 10.1.1.0/24)"""
        base = self.server.address.split("/")[0]
        parts = base.split(".")
        return f"{'.'.join(parts[:3])}.0/24"

    def add_peer(self, name: str, dns: str = DEFAULT_DNS,
                 mtu: int = DEFAULT_MTU, sync_remote: bool = True) -> Peer:
        """添加客户端"""
        if not self._server_id:
            raise RuntimeError("请先初始化或选择服务端")

        if self.db.get_peer_by_name(name, self._server_id):
            raise ValueError(f"客户端名称 '{name}' 已存在")

        private_key, public_key = generate_keypair()
        psk = generate_preshared_key()
        address = self._get_next_ip()

        # 客户端 AllowedIPs 默认为服务端网段
        allowed_ips = self._get_server_network()

        peer = Peer(
            id=None,
            server_id=self._server_id,
            name=name,
            public_key=public_key,
            private_key=private_key,
            preshared_key=psk,
            address=address,
            allowed_ips=allowed_ips,
            dns=dns,
            listen_port=generate_random_port(),
            mtu=mtu,
            created_at=datetime.now().isoformat(),
            enabled=True
        )
        peer = self.db.add_peer(peer)
        self._refresh_peers()

        # 同步到远程服务器
        if sync_remote:
            self._sync_peer_to_remote(peer, action="add")

        return peer

    def remove_peer(self, name: str, sync_remote: bool = True) -> bool:
        """删除客户端"""
        if not self._server_id:
            return False

        peer = self.db.get_peer_by_name(name, self._server_id)
        if not peer:
            return False

        # 保存 public_key 用于远程删除
        public_key = peer.public_key

        # 先从数据库删除
        if self.db.remove_peer(name, self._server_id):
            conf_file = self.export_dir / f"{name}.conf"
            if conf_file.exists():
                conf_file.unlink()
            self._refresh_peers()

            # 同步删除到远程
            if sync_remote:
                self._sync_peer_remove_to_remote(public_key)

            return True
        return False

    def toggle_peer(self, name: str, sync_remote: bool = True) -> Optional[bool]:
        """启用/禁用客户端"""
        if not self._server_id:
            return None

        result = self.db.toggle_peer(name, self._server_id)
        if result is not None:
            self._refresh_peers()
            if sync_remote:
                self._sync_to_remote()
        return result

    def import_existing_peer(self, name: str, public_key: str, address: str,
                             preshared_key: str = "") -> dict:
        """导入已有客户端（仅记录公钥，无法生成客户端配置）"""
        if not self._server_id:
            raise RuntimeError("请先初始化或选择服务端")

        if self.db.get_peer_by_name(name, self._server_id):
            raise ValueError(f"客户端名称 '{name}' 已存在")

        peer = Peer(
            id=None,
            server_id=self._server_id,
            name=name,
            public_key=public_key,
            private_key="",
            preshared_key=preshared_key,
            address=address,
            allowed_ips="",
            dns="",
            created_at=datetime.now().isoformat(),
            enabled=True
        )
        self.db.add_peer(peer)
        self._refresh_peers()
        return {"name": name, "public_key": public_key, "address": address, "imported": True}

    # ========== 配置生成 ==========

    def get_server_config(self) -> str:
        """生成服务端配置文件内容"""
        if not self.server.private_key:
            raise RuntimeError("服务端未初始化")

        config = f"""[Interface]
PrivateKey = {self.server.private_key}
Address = {self.server.address}
ListenPort = {self.server.listen_port}
"""
        if self.server.post_up:
            config += f"PostUp = {self.server.post_up}\n"
        if self.server.post_down:
            config += f"PostDown = {self.server.post_down}\n"

        for peer in self.peers:
            if peer.enabled:
                config += f"""
[Peer]
# {peer.name}
PublicKey = {peer.public_key}
PresharedKey = {peer.preshared_key}
AllowedIPs = {peer.address.split('/')[0]}/32
"""
        return config

    def get_client_config(self, name: str) -> str:
        """生成客户端配置文件内容"""
        if not self._server_id:
            raise RuntimeError("请先选择服务端")

        peer = self.db.get_peer_by_name(name, self._server_id)
        if not peer:
            raise ValueError(f"客户端 '{name}' 不存在")

        if not peer.private_key:
            raise ValueError(f"客户端 '{name}' 是导入的，没有私钥")

        if not self.server.endpoint:
            raise RuntimeError("服务端 endpoint 未配置")

        # 客户端地址使用 /24 网段
        address_ip = peer.address.split("/")[0]
        address_with_mask = f"{address_ip}/24"

        # 构建配置
        config = "# https://www.wireguard.com\n"
        config += "[Interface]\n"
        config += f"Address = {address_with_mask}\n"
        config += f"ListenPort = {peer.listen_port}\n"
        config += f"MTU = {peer.mtu}\n"
        config += f"PrivateKey = {peer.private_key}\n"

        if peer.dns:
            config += f"DNS = {peer.dns}\n"

        config += "\n[Peer]\n"
        # 客户端 AllowedIPs 使用服务端网段
        client_allowed_ips = self._get_server_network()
        config += f"AllowedIPs = {client_allowed_ips}\n"
        config += f"Endpoint = {self.server.endpoint}:{self.server.listen_port}\n"
        config += "PersistentKeepalive = 25\n"
        config += f"PresharedKey = {peer.preshared_key}\n"
        config += f"PublicKey = {self.server.public_key}\n"

        # 添加尾部注释
        config += f"# Client config --> {self.export_dir}/{name}.conf\n"

        return config

    # ========== 导出功能 ==========

    def export_client_config(self, name: str) -> Path:
        """导出客户端配置到文件"""
        config = self.get_client_config(name)
        filepath = self.export_dir / f"{name}.conf"
        with open(filepath, "w") as f:
            f.write(config)
        return filepath

    def export_client_qrcode(self, name: str) -> bool:
        """生成客户端配置二维码（终端显示）"""
        try:
            import qrcode
            config = self.get_client_config(name)
            qr = qrcode.QRCode(
                error_correction=qrcode.constants.ERROR_CORRECT_L,
                box_size=1,
                border=1,
            )
            qr.add_data(config)
            qr.make(fit=True)
            qr.print_ascii(invert=True)
            return True
        except Exception as e:
            print(f"生成二维码失败: {e}")
            return False

    def export_server_config(self) -> Path:
        """导出服务端配置到文件"""
        config = self.get_server_config()
        filepath = self.config_dir / f"{self.server.interface}.conf"
        with open(filepath, "w") as f:
            f.write(config)
        return filepath

    # ========== 远程同步 ==========

    def _sync_peer_to_remote(self, peer: Peer, action: str = "add") -> tuple[bool, str]:
        """同步添加客户端到远程服务器（不重启服务）"""
        remote_wg = self.get_remote_wg()
        if not remote_wg:
            return True, "SSH 未配置，跳过远程同步"

        # 1. 先更新配置文件（确保持久化）
        config = self.get_server_config()
        success, msg = remote_wg.update_config(config)
        if not success:
            return False, f"更新远程配置失败: {msg}"

        # 2. 动态添加 peer 到运行中的 WireGuard（不重启）
        allowed_ips = f"{peer.address.split('/')[0]}/32"
        success, msg = remote_wg.add_peer_live(
            peer.public_key, allowed_ips, peer.preshared_key
        )
        if success:
            return True, f"客户端 '{peer.name}' 已同步到远程"

        # 动态更新失败，尝试重载配置
        return remote_wg.reload()

    def _sync_peer_remove_to_remote(self, public_key: str) -> tuple[bool, str]:
        """同步删除客户端到远程服务器（不重启服务）"""
        remote_wg = self.get_remote_wg()
        if not remote_wg:
            return True, "SSH 未配置，跳过远程同步"

        # 1. 先更新配置文件（此时 peer 已从本地数据库删除）
        config = self.get_server_config()
        success, msg = remote_wg.update_config(config)
        if not success:
            return False, f"更新远程配置失败: {msg}"

        # 2. 动态移除 peer（不重启）
        success, msg = remote_wg.remove_peer_live(public_key)
        if success:
            return True, "客户端已从远程移除"

        # 动态更新失败，尝试重载配置
        return remote_wg.reload()

    def _sync_to_remote(self) -> tuple[bool, str]:
        """同步完整配置到远程服务器"""
        remote_wg = self.get_remote_wg()
        if not remote_wg:
            return True, "SSH 未配置，跳过远程同步"

        config = self.get_server_config()
        success, msg = remote_wg.update_config(config)
        if not success:
            return False, f"更新远程配置失败: {msg}"

        # 重载配置
        success, msg = remote_wg.reload()
        return success, msg

    def sync_to_remote(self) -> tuple[bool, str]:
        """手动同步到远程服务器"""
        return self._sync_to_remote()

    def get_remote_status(self) -> tuple[bool, str]:
        """获取远程 WireGuard 状态"""
        remote_wg = self.get_remote_wg()
        if not remote_wg:
            return False, "SSH 未配置"
        return remote_wg.get_status()

    # ========== 其他功能 ==========

    def list_peers(self) -> list[Peer]:
        """列出所有客户端"""
        return self.peers

    def get_status(self) -> dict:
        """获取本地 WireGuard 状态"""
        from .crypto import run_wg_command
        success, output = run_wg_command(["wg", "show"])
        return {"running": success, "output": output if success else "WireGuard 未运行或无权限"}



"""WireGuard 密钥生成模块"""

import subprocess
from typing import Optional


class WireGuardKeyError(Exception):
    """密钥操作错误"""
    pass


def run_wg_command(args: list[str]) -> tuple[bool, str]:
    """执行 wg 命令"""
    try:
        result = subprocess.run(
            args,
            capture_output=True,
            text=True,
            check=True
        )
        return True, result.stdout.strip()
    except subprocess.CalledProcessError as e:
        return False, e.stderr.strip()
    except FileNotFoundError:
        return False, "未找到 wg 命令，请确保已安装 WireGuard"


def generate_private_key() -> str:
    """生成私钥"""
    success, result = run_wg_command(["wg", "genkey"])
    if not success:
        raise WireGuardKeyError(f"生成私钥失败: {result}")
    return result


def generate_public_key(private_key: str) -> str:
    """从私钥生成公钥"""
    try:
        result = subprocess.run(
            ["wg", "pubkey"],
            input=private_key,
            capture_output=True,
            text=True,
            check=True
        )
        return result.stdout.strip()
    except Exception as e:
        raise WireGuardKeyError(f"生成公钥失败: {e}")


def generate_keypair() -> tuple[str, str]:
    """生成密钥对 (私钥, 公钥)"""
    private_key = generate_private_key()
    public_key = generate_public_key(private_key)
    return private_key, public_key


def generate_preshared_key() -> str:
    """生成预共享密钥"""
    success, result = run_wg_command(["wg", "genpsk"])
    if not success:
        raise WireGuardKeyError(f"生成预共享密钥失败: {result}")
    return result



"""命令行接口"""

import argparse
import sys

from .manager import WireGuardManager


class CancelInput(Exception):
    """用户取消输入"""
    pass


def get_input_required(prompt: str) -> str:
    """获取必填输入，Ctrl+C 取消"""
    try:
        value = input(f"{prompt}: ").strip()
        if not value:
            print("错误: 输入不能为空")
            raise CancelInput()
        return value
    except KeyboardInterrupt:
        print()
        raise CancelInput()


def get_input_optional(prompt: str, default: str = "") -> str:
    """获取可选输入，Ctrl+C 取消"""
    try:
        if default:
            value = input(f"{prompt} [{default}]: ").strip()
            return value if value else default
        else:
            return input(f"{prompt}: ").strip()
    except KeyboardInterrupt:
        print()
        raise CancelInput()


def interactive_menu(manager: WireGuardManager):
    """交互式菜单"""
    while True:
        # 清屏效果
        print("\n" * 2)

        # 标题
        print("┌" + "─" * 48 + "┐")
        print("│" + " " * 14 + "WireGuard 管理工具" + " " * 16 + "│")
        print("└" + "─" * 48 + "┘")

        # 状态栏
        servers = manager.get_servers()
        if servers:
            server_info = f"当前: {manager.server.endpoint}:{manager.server.interface}"
            peer_info = f"{len(manager.peers)} 个客户端"
            if len(servers) > 1:
                peer_info += f" | 共 {len(servers)} 个服务端"
        else:
            server_info = "未配置服务端"
            peer_info = ""

        ssh_icon = "●" if manager.get_ssh_client() else "○"
        ssh_status = "SSH 已连接" if manager.get_ssh_client() else "SSH 未配置"

        print(f"\n  {server_info}")
        if peer_info:
            print(f"  {peer_info}")
        print(f"  {ssh_icon} {ssh_status}")

        # 菜单
        print("\n┌─ 服务端 " + "─" * 39 + "┐")
        print("│  1. 初始化新服务端      2. 从文件导入          │")
        print("│  3. 手动输入私钥导入    4. 查看配置            │")
        print("│  5. 切换服务端          6. 删除服务端          │")
        print("├─ 客户端 " + "─" * 39 + "┤")
        print("│  7. 添加客户端          8. 删除客户端          │")
        print("│  9. 启用/禁用          10. 列出所有            │")
        print("│ 11. 导出配置           12. 显示二维码          │")
        print("├─ 远程管理 " + "─" * 37 + "┤")
        print("│ 13. 配置 SSH           14. 从远程导入          │")
        print("│ 15. 同步到远程         16. 查看远程状态        │")
        print("├─ 其他 " + "─" * 41 + "┤")
        print("│ 17. 导出服务端配置      0. 退出                │")
        print("└" + "─" * 48 + "┘")
        try:
            choice = input("请选择 [0-17]: ").strip()
        except KeyboardInterrupt:
            print("\n再见!")
            break

        try:
            if choice == "0":
                print("再见!")
                break

            elif choice == "1":
                _menu_init_server(manager)

            elif choice == "2":
                _menu_import_from_file(manager)

            elif choice == "3":
                _menu_import_manual(manager)

            elif choice == "4":
                _menu_show_server(manager)

            elif choice == "5":
                _menu_switch_server(manager)

            elif choice == "6":
                _menu_delete_server(manager)

            elif choice == "7":
                _menu_add_peer(manager)

            elif choice == "8":
                _menu_remove_peer(manager)

            elif choice == "9":
                _menu_toggle_peer(manager)

            elif choice == "10":
                _menu_list_peers(manager)

            elif choice == "11":
                _menu_export_peer(manager)

            elif choice == "12":
                _menu_show_qrcode(manager)

            elif choice == "13":
                _menu_setup_ssh(manager)

            elif choice == "14":
                _menu_import_from_remote(manager)

            elif choice == "15":
                _menu_sync_remote(manager)

            elif choice == "16":
                _menu_remote_status(manager)

            elif choice == "17":
                _menu_export_server(manager)

            else:
                print("无效选择")

        except CancelInput:
            print("\n已取消")
        except Exception as e:
            print(f"错误: {e}")


def _menu_init_server(manager: WireGuardManager):
    print("\n--- 初始化新服务端 --- (Ctrl+C 返回菜单)")
    endpoint = get_input_required("公网 IP 或域名")
    address = get_input_optional("服务端内网地址", "10.0.0.1/24")
    port = get_input_optional("监听端口", "51820")
    interface = get_input_optional("接口名称", "wg0")

    server = manager.init_server(endpoint, address, int(port), interface)
    print(f"\n✓ 服务端初始化成功!")
    print(f"  公钥: {server.public_key}")


def _menu_import_from_file(manager: WireGuardManager):
    print("\n--- 从配置文件导入服务端 --- (Ctrl+C 返回菜单)")
    config_path = get_input_required("配置文件路径 (如 /etc/wireguard/wg0.conf)")
    endpoint = get_input_required("公网 IP 或域名")

    server, existing_peers = manager.import_server_from_config(config_path, endpoint)
    print(f"\n✓ 服务端导入成功!")
    print(f"  公钥: {server.public_key}")
    print(f"  地址: {server.address}")
    print(f"  端口: {server.listen_port}")

    _handle_existing_peers(manager, existing_peers)


def _menu_import_manual(manager: WireGuardManager):
    print("\n--- 手动导入服务端 --- (Ctrl+C 返回菜单)")
    private_key = get_input_required("服务端私钥")
    endpoint = get_input_required("公网 IP 或域名")
    address = get_input_optional("服务端内网地址", "10.0.0.1/24")
    port = get_input_optional("监听端口", "51820")
    interface = get_input_optional("接口名称", "wg0")

    server = manager.import_server(private_key, endpoint, address, int(port), interface)
    print(f"\n✓ 服务端导入成功!")
    print(f"  公钥: {server.public_key}")


def _menu_show_server(manager: WireGuardManager):
    if not manager.server.private_key:
        print("错误: 服务端未初始化")
        return
    print("\n--- 服务端配置 ---")
    print(f"公钥: {manager.server.public_key}")
    print(f"地址: {manager.server.address}")
    print(f"端口: {manager.server.listen_port}")
    print(f"Endpoint: {manager.server.endpoint}")
    print(f"接口: {manager.server.interface}")
    print(f"客户端数: {len(manager.peers)}")


def _menu_switch_server(manager: WireGuardManager):
    servers = manager.get_servers()
    if not servers:
        print("没有可用的服务端")
        return
    if len(servers) == 1:
        print("只有一个服务端，无需切换")
        return

    print("\n--- 切换服务端 --- (Ctrl+C 返回菜单)")
    for i, s in enumerate(servers, 1):
        current = " (当前)" if s.id == manager.server_id else ""
        peer_count = len(manager.db.get_peers(s.id))
        print(f"  {i}. {s.endpoint}:{s.interface} - {s.address} Port:{s.listen_port} ({peer_count} 客户端){current}")

    choice = get_input_required("选择服务端序号")
    try:
        idx = int(choice) - 1
        if 0 <= idx < len(servers):
            if manager.switch_server(servers[idx].id):
                print(f"✓ 已切换到服务端: {manager.server.endpoint}")
            else:
                print("切换失败")
        else:
            print("无效选择")
    except ValueError:
        print("请输入数字")


def _menu_delete_server(manager: WireGuardManager):
    servers = manager.get_servers()
    if not servers:
        print("没有可用的服务端")
        return

    print("\n--- 删除服务端 --- (Ctrl+C 返回菜单)")
    for i, s in enumerate(servers, 1):
        current = " (当前)" if s.id == manager.server_id else ""
        peer_count = len(manager.db.get_peers(s.id))
        print(f"  {i}. {s.endpoint}:{s.interface} - {s.address} Port:{s.listen_port} ({peer_count} 客户端){current}")

    choice = get_input_required("选择要删除的服务端序号")
    try:
        idx = int(choice) - 1
        if 0 <= idx < len(servers):
            server = servers[idx]
            confirm = get_input_optional(f"确定删除 '{server.endpoint}:{server.interface}' 及其所有客户端? (y/n)", "n")
            if confirm.lower() == 'y':
                if manager.delete_server(server.id):
                    print(f"✓ 已删除服务端: {server.endpoint}")
                else:
                    print("删除失败")
        else:
            print("无效选择")
    except ValueError:
        print("请输入数字")


def _menu_add_peer(manager: WireGuardManager):
    if not manager.server.private_key:
        print("错误: 请先初始化服务端")
        return
    print("\n--- 添加客户端 --- (Ctrl+C 返回菜单)")
    name = get_input_required("客户端名称")
    dns = get_input_optional("DNS 服务器 (留空不设置)")
    mtu = get_input_optional("MTU", "1280")

    peer = manager.add_peer(name, dns, int(mtu))
    print(f"\n✓ 客户端 '{name}' 添加成功!")
    print(f"  分配 IP: {peer.address}")
    print(f"  监听端口: {peer.listen_port}")

    # 自动显示客户端配置
    print("\n--- 客户端配置 ---")
    print(manager.get_client_config(name))


def _menu_remove_peer(manager: WireGuardManager):
    if not manager.peers:
        print("没有客户端")
        return
    print("\n--- 删除客户端 --- (Ctrl+C 返回菜单)")
    for i, p in enumerate(manager.peers, 1):
        print(f"  {i}. {p.name} ({p.address})")
    name = get_input_required("输入客户端名称")
    if manager.remove_peer(name):
        print(f"✓ 客户端 '{name}' 已删除")
    else:
        print(f"错误: 客户端 '{name}' 不存在")


def _menu_toggle_peer(manager: WireGuardManager):
    if not manager.peers:
        print("没有客户端")
        return
    print("\n--- 启用/禁用客户端 --- (Ctrl+C 返回菜单)")
    for p in manager.peers:
        status = "✓ 启用" if p.enabled else "✗ 禁用"
        print(f"  {p.name}: {status}")
    name = get_input_required("输入客户端名称")
    result = manager.toggle_peer(name)
    if result is not None:
        status = "启用" if result else "禁用"
        print(f"✓ 客户端 '{name}' 已{status}")
    else:
        print(f"错误: 客户端 '{name}' 不存在")


def _menu_list_peers(manager: WireGuardManager):
    print(f"\n--- 客户端列表 [{manager.server.endpoint}] ---")
    if not manager.peers:
        print("  (空)")
    else:
        for p in manager.peers:
            status = "✓" if p.enabled else "✗"
            imported = " [导入]" if not p.private_key else ""
            print(f"  {status} {p.name}{imported}")
            print(f"      IP: {p.address}")
            print(f"      监听端口: {p.listen_port}  MTU: {p.mtu}")
            print(f"      公钥: {p.public_key[:20]}...")
            print(f"      创建: {p.created_at[:10]}")


def _menu_export_peer(manager: WireGuardManager):
    if not manager.peers:
        print("没有客户端")
        return
    exportable = [p for p in manager.peers if p.private_key]
    if not exportable:
        print("没有可导出的客户端 (导入的客户端没有私钥)")
        return
    print("\n--- 导出客户端配置 --- (Ctrl+C 返回菜单)")
    for p in exportable:
        print(f"  - {p.name}")
    name = get_input_required("输入客户端名称")

    peer = manager.db.get_peer_by_name(name, manager.server_id)
    if not peer:
        print(f"错误: 客户端 '{name}' 不存在")
        return
    if not peer.private_key:
        print(f"错误: '{name}' 是导入的客户端，没有私钥，无法导出配置")
        return

    # 显示配置内容
    print("\n--- 配置内容 ---")
    print(manager.get_client_config(name))

    # 询问是否保存到文件
    save = get_input_optional("保存到文件? (y/n)", "n")
    if save.lower() == 'y':
        path = manager.export_client_config(name)
        print(f"✓ 已保存到: {path}")


def _menu_show_qrcode(manager: WireGuardManager):
    if not manager.peers:
        print("没有客户端")
        return
    exportable = [p for p in manager.peers if p.private_key]
    if not exportable:
        print("没有可导出的客户端 (导入的客户端没有私钥)")
        return
    print("\n--- 显示客户端二维码 --- (Ctrl+C 返回菜单)")
    for p in exportable:
        print(f"  - {p.name}")
    name = get_input_required("输入客户端名称")
    peer = manager.db.get_peer_by_name(name, manager.server_id)
    if peer and not peer.private_key:
        print(f"错误: '{name}' 是导入的客户端，没有私钥，无法生成二维码")
        return
    print(f"\n{name} 的配置二维码:\n")
    manager.export_client_qrcode(name)


def _menu_setup_ssh(manager: WireGuardManager):
    print("\n--- 配置 SSH 连接 --- (Ctrl+C 返回菜单)")
    host = get_input_required("服务器地址")
    port = get_input_optional("SSH 端口", "22")
    user = get_input_optional("用户名", "root")

    print(f"\n正在测试连接 {user}@{host}:{port} ...")
    success, msg = manager.setup_ssh(host, int(port), user)
    if success:
        print(f"✓ SSH 连接成功!")
    else:
        print(f"✗ SSH 连接失败: {msg}")


def _menu_import_from_remote(manager: WireGuardManager):
    if not manager.get_ssh_client():
        print("错误: 请先配置 SSH 连接")
        return
    print("\n--- 从远程服务器导入配置 --- (Ctrl+C 返回菜单)")
    endpoint = get_input_required("公网 IP 或域名")

    server, existing_peers = manager.import_server_from_remote(endpoint)
    print(f"\n✓ 服务端导入成功!")
    print(f"  公钥: {server.public_key}")
    print(f"  地址: {server.address}")
    print(f"  端口: {server.listen_port}")

    _handle_existing_peers(manager, existing_peers)


def _menu_sync_remote(manager: WireGuardManager):
    if not manager.get_ssh_client():
        print("错误: 请先配置 SSH 连接")
        return
    print("\n正在同步配置到远程服务器...")
    success, msg = manager.sync_to_remote()
    if success:
        print(f"✓ {msg}")
    else:
        print(f"✗ {msg}")


def _menu_remote_status(manager: WireGuardManager):
    if not manager.get_ssh_client():
        print("错误: 请先配置 SSH 连接")
        return
    print("\n--- 远程 WireGuard 状态 ---")
    success, output = manager.get_remote_status()
    print(output)


def _menu_export_server(manager: WireGuardManager):
    path = manager.export_server_config()
    print(f"\n✓ 服务端配置已导出到: {path}")
    print("\n使用以下命令部署到服务器:")
    print(f"  sudo cp {path} /etc/wireguard/")
    print(f"  sudo systemctl enable wg-quick@{manager.server.interface}")
    print(f"  sudo systemctl start wg-quick@{manager.server.interface}")


def _handle_existing_peers(manager: WireGuardManager, existing_peers: list[dict]):
    """处理导入时发现的已有客户端"""
    if existing_peers:
        print(f"\n发现 {len(existing_peers)} 个已有客户端:")
        for p in existing_peers:
            print(f"  - {p['name']}: {p['allowed_ips']}")
        import_choice = input("\n是否导入这些客户端? (y/n) [n]: ").strip().lower()
        if import_choice == 'y':
            for p in existing_peers:
                try:
                    manager.import_existing_peer(
                        name=p['name'],
                        public_key=p['public_key'],
                        address=p['allowed_ips'],
                        preshared_key=p.get('preshared_key', '')
                    )
                    print(f"  ✓ 导入 {p['name']}")
                except Exception as e:
                    print(f"  ✗ 导入 {p['name']} 失败: {e}")
            print("\n注意: 导入的客户端没有私钥，无法生成客户端配置文件")


def create_parser() -> argparse.ArgumentParser:
    """创建命令行参数解析器"""
    parser = argparse.ArgumentParser(
        description="WireGuard 管理工具",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog="""
示例:
  %(prog)s                                      # 交互式菜单
  %(prog)s servers                              # 列出所有服务端
  %(prog)s use example.com                      # 切换到指定服务端
  %(prog)s use example.com -i wg1               # 切换到指定服务端的 wg1 接口
  %(prog)s init -e example.com                  # 初始化新服务端 (wg0)
  %(prog)s init -e example.com -i wg1 -a 10.1.0.1/24  # 初始化同一服务器的 wg1 接口
  %(prog)s import -f /etc/wireguard/wg0.conf -e example.com  # 从配置文件导入
  %(prog)s import -k <私钥> -e example.com      # 手动输入私钥导入
  %(prog)s add -n phone                         # 添加客户端
  %(prog)s add -n phone --dns 1.1.1.1 --mtu 1420  # 添加客户端 (指定 DNS 和 MTU)
  %(prog)s list                                 # 列出客户端
  %(prog)s export -n phone                      # 显示客户端配置
  %(prog)s export -n phone --save               # 显示并保存到文件
  %(prog)s export -n phone --qr                 # 显示二维码
  %(prog)s server                               # 导出服务端配置
  %(prog)s ssh --host 1.2.3.4                   # 配置 SSH
  %(prog)s sync                                 # 同步到远程服务器
"""
    )

    # 全局选项
    parser.add_argument("-s", "--server", help="指定服务端 (endpoint)")

    subparsers = parser.add_subparsers(dest="command", help="命令")

    # servers 命令
    subparsers.add_parser("servers", help="列出所有服务端")

    # use 命令
    use_parser = subparsers.add_parser("use", help="切换服务端")
    use_parser.add_argument("endpoint", help="服务端 endpoint")
    use_parser.add_argument("-i", "--interface", help="接口名称 (同一 endpoint 多接口时使用)")

    # init 命令
    init_parser = subparsers.add_parser("init", help="初始化新服务端")
    init_parser.add_argument("-e", "--endpoint", required=True, help="公网 IP 或域名")
    init_parser.add_argument("-a", "--address", default="10.0.0.1/24", help="服务端地址")
    init_parser.add_argument("-p", "--port", type=int, default=51820, help="监听端口")
    init_parser.add_argument("-i", "--interface", default="wg0", help="接口名称")

    # import 命令
    import_parser = subparsers.add_parser("import", help="导入已有服务端")
    import_parser.add_argument("-e", "--endpoint", required=True, help="公网 IP 或域名")
    import_group = import_parser.add_mutually_exclusive_group(required=True)
    import_group.add_argument("-f", "--file", help="配置文件路径")
    import_group.add_argument("-k", "--private-key", help="服务端私钥")
    import_group.add_argument("--remote", action="store_true", help="从远程服务器导入")
    import_parser.add_argument("-a", "--address", default="10.0.0.1/24", help="服务端地址")
    import_parser.add_argument("-p", "--port", type=int, default=51820, help="监听端口")
    import_parser.add_argument("-i", "--interface", default="wg0", help="接口名称")
    import_parser.add_argument("--import-peers", action="store_true", help="同时导入客户端")

    # delete-server 命令
    delete_server_parser = subparsers.add_parser("delete-server", help="删除服务端")
    delete_server_parser.add_argument("endpoint", help="服务端 endpoint")
    delete_server_parser.add_argument("-y", "--yes", action="store_true", help="跳过确认")

    # add 命令
    add_parser = subparsers.add_parser("add", help="添加客户端")
    add_parser.add_argument("-n", "--name", required=True, help="客户端名称")
    add_parser.add_argument("--dns", default="", help="DNS 服务器 (留空不设置)")
    add_parser.add_argument("--mtu", type=int, default=1280, help="MTU (默认 1280)")
    add_parser.add_argument("--no-sync", action="store_true", help="不同步到远程")

    # remove 命令
    remove_parser = subparsers.add_parser("remove", help="删除客户端")
    remove_parser.add_argument("-n", "--name", required=True, help="客户端名称")
    remove_parser.add_argument("--no-sync", action="store_true", help="不同步到远程")

    # list 命令
    subparsers.add_parser("list", help="列出客户端")

    # export 命令
    export_parser = subparsers.add_parser("export", help="导出客户端配置")
    export_parser.add_argument("-n", "--name", required=True, help="客户端名称")
    export_parser.add_argument("--qr", action="store_true", help="显示二维码")
    export_parser.add_argument("--save", action="store_true", help="保存到文件")

    # server 命令
    subparsers.add_parser("server", help="导出服务端配置")

    # show 命令
    subparsers.add_parser("show", help="显示服务端配置信息")

    # ssh 命令
    ssh_parser = subparsers.add_parser("ssh", help="配置 SSH 连接")
    ssh_parser.add_argument("--host", required=True, help="服务器地址")
    ssh_parser.add_argument("--port", type=int, default=22, help="SSH 端口")
    ssh_parser.add_argument("--user", default="root", help="用户名")

    # sync 命令
    subparsers.add_parser("sync", help="同步配置到远程服务器")

    # remote-status 命令
    subparsers.add_parser("remote-status", help="查看远程 WireGuard 状态")

    return parser


def main():
    """主入口"""
    import signal

    # 设置信号处理，优雅退出
    def signal_handler(sig, frame):
        print("\n再见!")
        sys.exit(0)

    signal.signal(signal.SIGINT, signal_handler)

    parser = create_parser()
    args = parser.parse_args()

    # 根据 -s 参数选择服务端
    if hasattr(args, 'server') and args.server:
        manager = WireGuardManager()
        if not manager.switch_server_by_endpoint(args.server):
            print(f"错误: 服务端 '{args.server}' 不存在", file=sys.stderr)
            sys.exit(1)
    else:
        manager = WireGuardManager()

    if not args.command:
        interactive_menu(manager)
        return

    try:
        if args.command == "servers":
            servers = manager.get_servers()
            if not servers:
                print("没有服务端")
            else:
                print("服务端列表:")
                for s in servers:
                    current = " (当前)" if s.id == manager.server_id else ""
                    peer_count = len(manager.db.get_peers(s.id))
                    print(f"  {s.endpoint}:{s.interface}\t{s.address}\tPort:{s.listen_port}\t{peer_count} 客户端{current}")

        elif args.command == "use":
            if manager.switch_server_by_endpoint(args.endpoint, args.interface):
                if args.interface:
                    print(f"已切换到服务端: {args.endpoint}:{args.interface}")
                else:
                    print(f"已切换到服务端: {args.endpoint}:{manager.server.interface}")
            else:
                if args.interface:
                    print(f"错误: 服务端 '{args.endpoint}:{args.interface}' 不存在", file=sys.stderr)
                else:
                    print(f"错误: 服务端 '{args.endpoint}' 不存在", file=sys.stderr)
                sys.exit(1)

        elif args.command == "init":
            server = manager.init_server(args.endpoint, args.address, args.port, args.interface)
            print(f"服务端初始化成功!")
            print(f"公钥: {server.public_key}")

        elif args.command == "import":
            if args.file:
                server, existing_peers = manager.import_server_from_config(args.file, args.endpoint)
            elif args.remote:
                server, existing_peers = manager.import_server_from_remote(args.endpoint)
            else:
                server = manager.import_server(
                    args.private_key, args.endpoint, args.address, args.port, args.interface
                )
                existing_peers = []

            print(f"服务端导入成功!")
            print(f"公钥: {server.public_key}")

            if existing_peers:
                print(f"\n发现 {len(existing_peers)} 个已有客户端")
                if args.import_peers:
                    for p in existing_peers:
                        try:
                            manager.import_existing_peer(
                                name=p['name'],
                                public_key=p['public_key'],
                                address=p['allowed_ips'],
                                preshared_key=p.get('preshared_key', '')
                            )
                            print(f"  ✓ 导入 {p['name']}")
                        except Exception as e:
                            print(f"  ✗ 导入 {p['name']} 失败: {e}")

        elif args.command == "delete-server":
            server = manager.db.get_server_by_endpoint(args.endpoint)
            if not server:
                print(f"错误: 服务端 '{args.endpoint}' 不存在", file=sys.stderr)
                sys.exit(1)

            if not args.yes:
                peer_count = len(manager.db.get_peers(server.id))
                confirm = input(f"确定删除 '{args.endpoint}' 及其 {peer_count} 个客户端? (y/n) [n]: ").strip().lower()
                if confirm != 'y':
                    print("已取消")
                    sys.exit(0)

            if manager.delete_server(server.id):
                print(f"已删除服务端: {args.endpoint}")
            else:
                print("删除失败", file=sys.stderr)
                sys.exit(1)

        elif args.command == "add":
            peer = manager.add_peer(args.name, args.dns, args.mtu,
                                    sync_remote=not args.no_sync)
            print(f"客户端 '{args.name}' 添加成功!")
            print(f"IP: {peer.address}")
            print(f"监听端口: {peer.listen_port}")
            # 自动显示客户端配置
            print("\n--- 客户端配置 ---")
            print(manager.get_client_config(args.name))

        elif args.command == "remove":
            if manager.remove_peer(args.name, sync_remote=not args.no_sync):
                print(f"客户端 '{args.name}' 已删除")
            else:
                print(f"错误: 客户端 '{args.name}' 不存在", file=sys.stderr)
                sys.exit(1)

        elif args.command == "list":
            if not manager.peers:
                print(f"没有客户端 [{manager.server.endpoint}]")
            else:
                print(f"客户端列表 [{manager.server.endpoint}]:")
                for p in manager.peers:
                    status = "✓" if p.enabled else "✗"
                    imported = " [导入]" if not p.private_key else ""
                    port_info = f"Port:{p.listen_port}" if p.listen_port else ""
                    print(f"{status} {p.name}{imported}\t{p.address}\t{port_info}\tMTU:{p.mtu}\t{p.created_at[:10]}")

        elif args.command == "export":
            peer = manager.db.get_peer_by_name(args.name, manager.server_id)
            if not peer:
                print(f"错误: 客户端 '{args.name}' 不存在", file=sys.stderr)
                sys.exit(1)
            if not peer.private_key:
                print(f"错误: '{args.name}' 是导入的客户端，没有私钥", file=sys.stderr)
                sys.exit(1)
            if args.qr:
                manager.export_client_qrcode(args.name)
            else:
                print(manager.get_client_config(args.name))
                if args.save:
                    path = manager.export_client_config(args.name)
                    print(f"\n已保存到: {path}", file=sys.stderr)

        elif args.command == "server":
            path = manager.export_server_config()
            print(f"服务端配置已导出: {path}")
            print("\n" + manager.get_server_config())

        elif args.command == "show":
            if not manager.server.private_key:
                print("服务端未初始化", file=sys.stderr)
                sys.exit(1)
            print(f"接口: {manager.server.interface}")
            print(f"公钥: {manager.server.public_key}")
            print(f"地址: {manager.server.address}")
            print(f"端口: {manager.server.listen_port}")
            print(f"Endpoint: {manager.server.endpoint}")
            print(f"客户端数: {len(manager.peers)}")

        elif args.command == "ssh":
            success, msg = manager.setup_ssh(args.host, args.port, args.user)
            if success:
                print(f"SSH 连接配置成功!")
            else:
                print(f"SSH 连接失败: {msg}", file=sys.stderr)
                sys.exit(1)

        elif args.command == "sync":
            success, msg = manager.sync_to_remote()
            if success:
                print(msg)
            else:
                print(f"同步失败: {msg}", file=sys.stderr)
                sys.exit(1)

        elif args.command == "remote-status":
            success, output = manager.get_remote_status()
            if success:
                print(output)
            else:
                print(f"获取远程状态失败: {output}", file=sys.stderr)
                sys.exit(1)

    except Exception as e:
        print(f"错误: {e}", file=sys.stderr)
        sys.exit(1)


if __name__ == "__main__":
    main()



"""数据模型定义"""

from dataclasses import dataclass
from typing import Optional


@dataclass
class Peer:
    """客户端/Peer 配置"""
    id: Optional[int]
    server_id: int  # 所属服务端 ID
    name: str
    public_key: str
    private_key: str
    preshared_key: str
    address: str
    allowed_ips: str = "0.0.0.0/0, ::/0"
    dns: str = ""
    listen_port: int = 0  # 客户端监听端口，0 表示随机
    mtu: int = 1280
    created_at: str = ""
    enabled: bool = True


@dataclass
class ServerConfig:
    """服务端配置"""
    id: Optional[int] = None
    private_key: str = ""
    public_key: str = ""
    address: str = "10.0.0.1/24"
    listen_port: int = 51820
    interface: str = "wg0"
    endpoint: str = ""
    post_up: str = ""
    post_down: str = ""



"""配置常量"""

import random
from pathlib import Path

# 本地配置目录
CONFIG_DIR = Path.home() / ".wg_manager"
DB_FILE = CONFIG_DIR / "wg_manager.db"
EXPORT_DIR = CONFIG_DIR / "clients"

# WireGuard 默认配置
DEFAULT_ADDRESS = "10.0.0.1/24"
DEFAULT_PORT = 51820
DEFAULT_INTERFACE = "wg0"
DEFAULT_DNS = ""  # 默认不设置 DNS
DEFAULT_MTU = 1280

# 远程服务器 WireGuard 配置路径
REMOTE_WG_DIR = "/etc/wireguard"


def generate_random_port() -> int:
    """生成随机监听端口 (10000-60000)"""
    return random.randint(10000, 60000)



"""SSH 远程管理模块"""

import subprocess
from dataclasses import dataclass
from pathlib import Path
from typing import Optional

from .config import REMOTE_WG_DIR


@dataclass
class SSHConfig:
    """SSH 配置"""
    host: str
    port: int = 22
    user: str = "root"
    key_file: Optional[str] = None


class SSHClient:
    """SSH 客户端 - 使用系统 ssh 命令"""

    def __init__(self, config: SSHConfig):
        self.config = config
        self._connected = False

    def _build_ssh_cmd(self, extra_args: list[str] = None) -> list[str]:
        """构建 SSH 命令"""
        cmd = [
            "ssh",
            "-o", "StrictHostKeyChecking=no",
            "-o", "BatchMode=yes",
            "-o", "ConnectTimeout=10"
        ]

        if self.config.port != 22:
            cmd.extend(["-p", str(self.config.port)])

        if self.config.key_file:
            cmd.extend(["-i", self.config.key_file])

        cmd.append(f"{self.config.user}@{self.config.host}")

        if extra_args:
            cmd.extend(extra_args)

        return cmd

    def _build_scp_cmd(self, local_path: str, remote_path: str, upload: bool = True) -> list[str]:
        """构建 SCP 命令"""
        cmd = [
            "scp",
            "-o", "StrictHostKeyChecking=no",
            "-o", "BatchMode=yes",
            "-o", "ConnectTimeout=10"
        ]

        if self.config.port != 22:
            cmd.extend(["-P", str(self.config.port)])

        if self.config.key_file:
            cmd.extend(["-i", self.config.key_file])

        remote = f"{self.config.user}@{self.config.host}:{remote_path}"

        if upload:
            cmd.extend([local_path, remote])
        else:
            cmd.extend([remote, local_path])

        return cmd

    def test_connection(self) -> tuple[bool, str]:
        """测试 SSH 连接"""
        try:
            cmd = self._build_ssh_cmd(["echo", "ok"])
            result = subprocess.run(cmd, capture_output=True, text=True, timeout=10)
            if result.returncode == 0:
                self._connected = True
                return True, "连接成功"
            return False, result.stderr.strip() or "连接失败"
        except subprocess.TimeoutExpired:
            return False, "连接超时"
        except Exception as e:
            return False, str(e)

    def run_command(self, command: str) -> tuple[bool, str]:
        """执行远程命令"""
        try:
            cmd = self._build_ssh_cmd([command])
            result = subprocess.run(cmd, capture_output=True, text=True, timeout=30)
            if result.returncode == 0:
                return True, result.stdout.strip()
            return False, result.stderr.strip() or result.stdout.strip()
        except subprocess.TimeoutExpired:
            return False, "命令执行超时"
        except Exception as e:
            return False, str(e)

    def upload_file(self, local_path: str, remote_path: str) -> tuple[bool, str]:
        """上传文件到远程服务器"""
        try:
            cmd = self._build_scp_cmd(local_path, remote_path, upload=True)
            result = subprocess.run(cmd, capture_output=True, text=True, timeout=30)
            if result.returncode == 0:
                return True, "上传成功"
            return False, result.stderr.strip()
        except subprocess.TimeoutExpired:
            return False, "上传超时"
        except Exception as e:
            return False, str(e)

    def download_file(self, remote_path: str, local_path: str) -> tuple[bool, str]:
        """从远程服务器下载文件"""
        try:
            cmd = self._build_scp_cmd(local_path, remote_path, upload=False)
            result = subprocess.run(cmd, capture_output=True, text=True, timeout=30)
            if result.returncode == 0:
                return True, "下载成功"
            return False, result.stderr.strip()
        except subprocess.TimeoutExpired:
            return False, "下载超时"
        except Exception as e:
            return False, str(e)

    def read_remote_file(self, remote_path: str) -> tuple[bool, str]:
        """读取远程文件内容"""
        return self.run_command(f"cat {remote_path}")

    def write_remote_file(self, remote_path: str, content: str) -> tuple[bool, str]:
        """写入远程文件（通过 stdin）"""
        try:
            cmd = self._build_ssh_cmd([f"cat > {remote_path}"])
            result = subprocess.run(
                cmd, input=content, capture_output=True, text=True, timeout=30
            )
            if result.returncode == 0:
                return True, "写入成功"
            return False, result.stderr.strip()
        except Exception as e:
            return False, str(e)


class RemoteWireGuard:
    """远程 WireGuard 管理"""

    def __init__(self, ssh_client: SSHClient, interface: str = "wg0"):
        self.ssh = ssh_client
        self.interface = interface
        self.config_path = f"{REMOTE_WG_DIR}/{interface}.conf"

    def get_config(self) -> tuple[bool, str]:
        """获取远程配置文件"""
        return self.ssh.read_remote_file(self.config_path)

    def update_config(self, config_content: str) -> tuple[bool, str]:
        """更新远程配置文件"""
        return self.ssh.write_remote_file(self.config_path, config_content)

    def get_status(self) -> tuple[bool, str]:
        """获取 WireGuard 状态"""
        return self.ssh.run_command("wg show")

    def restart(self) -> tuple[bool, str]:
        """重启 WireGuard 服务"""
        success, output = self.ssh.run_command(
            f"systemctl restart wg-quick@{self.interface}"
        )
        if success:
            return True, f"WireGuard {self.interface} 已重启"
        return False, output

    def reload(self) -> tuple[bool, str]:
        """重载配置（不中断连接）"""
        # 先检查接口是否存在
        check_success, _ = self.ssh.run_command(f"ip link show {self.interface}")

        if check_success:
            # 接口存在，使用 wg syncconf 热重载
            # 先生成 strip 配置到临时文件，避免进程替换问题
            strip_cmd = (
                f"wg-quick strip {self.interface} > /tmp/{self.interface}_strip.conf && "
                f"wg syncconf {self.interface} /tmp/{self.interface}_strip.conf && "
                f"rm -f /tmp/{self.interface}_strip.conf"
            )
            success, output = self.ssh.run_command(strip_cmd)
            if success:
                return True, "配置已重载"
            # syncconf 失败，尝试重启
            return self.restart()
        else:
            # 接口不存在，需要启动服务
            return self.restart()

    def start(self) -> tuple[bool, str]:
        """启动 WireGuard"""
        return self.ssh.run_command(f"systemctl start wg-quick@{self.interface}")

    def stop(self) -> tuple[bool, str]:
        """停止 WireGuard"""
        return self.ssh.run_command(f"systemctl stop wg-quick@{self.interface}")

    def enable(self) -> tuple[bool, str]:
        """设置开机启动"""
        return self.ssh.run_command(f"systemctl enable wg-quick@{self.interface}")

    def is_active(self) -> bool:
        """检查服务是否运行"""
        success, output = self.ssh.run_command(
            f"systemctl is-active wg-quick@{self.interface}"
        )
        return success and output.strip() == "active"

    def add_peer_live(self, public_key: str, allowed_ips: str,
                      preshared_key: str = "") -> tuple[bool, str]:
        """动态添加 peer（不重启服务）"""
        cmd = f"wg set {self.interface} peer {public_key} allowed-ips {allowed_ips}"
        if preshared_key:
            # 需要通过临时文件传递 preshared key
            cmd = f"echo '{preshared_key}' | wg set {self.interface} peer {public_key} preshared-key /dev/stdin allowed-ips {allowed_ips}"
        return self.ssh.run_command(cmd)

    def remove_peer_live(self, public_key: str) -> tuple[bool, str]:
        """动态移除 peer（不重启服务）"""
        return self.ssh.run_command(
            f"wg set {self.interface} peer {public_key} remove"
        )



"""SQLite 数据库管理"""

import sqlite3
from pathlib import Path
from typing import Optional

from .models import Peer, ServerConfig


class Database:
    """SQLite 数据库管理"""

    def __init__(self, db_path: Path):
        self.db_path = db_path
        self.db_path.parent.mkdir(parents=True, exist_ok=True)
        self._init_db()
        self._migrate_db()

    def _get_conn(self) -> sqlite3.Connection:
        conn = sqlite3.connect(self.db_path)
        conn.row_factory = sqlite3.Row
        return conn

    def _init_db(self):
        """初始化数据库表"""
        with self._get_conn() as conn:
            conn.execute("""
                CREATE TABLE IF NOT EXISTS server (
                    id INTEGER PRIMARY KEY AUTOINCREMENT,
                    private_key TEXT NOT NULL,
                    public_key TEXT NOT NULL,
                    address TEXT NOT NULL DEFAULT '10.0.0.1/24',
                    listen_port INTEGER NOT NULL DEFAULT 51820,
                    interface TEXT NOT NULL DEFAULT 'wg0',
                    endpoint TEXT NOT NULL,
                    post_up TEXT DEFAULT '',
                    post_down TEXT DEFAULT '',
                    ssh_host TEXT DEFAULT '',
                    ssh_port INTEGER DEFAULT 22,
                    ssh_user TEXT DEFAULT 'root'
                )
            """)
            conn.execute("""
                CREATE TABLE IF NOT EXISTS peers (
                    id INTEGER PRIMARY KEY AUTOINCREMENT,
                    server_id INTEGER NOT NULL DEFAULT 1,
                    name TEXT NOT NULL,
                    public_key TEXT NOT NULL,
                    private_key TEXT DEFAULT '',
                    preshared_key TEXT DEFAULT '',
                    address TEXT NOT NULL,
                    allowed_ips TEXT DEFAULT '',
                    dns TEXT DEFAULT '',
                    listen_port INTEGER DEFAULT 0,
                    mtu INTEGER DEFAULT 1280,
                    created_at TEXT NOT NULL,
                    enabled INTEGER DEFAULT 1,
                    UNIQUE(server_id, name),
                    FOREIGN KEY (server_id) REFERENCES server(id)
                )
            """)
            conn.commit()

    def _migrate_db(self):
        """数据库迁移 - 添加新字段"""
        with self._get_conn() as conn:
            # 检查 peers 表字段
            cursor = conn.execute("PRAGMA table_info(peers)")
            columns = [row[1] for row in cursor.fetchall()]

            if "listen_port" not in columns:
                conn.execute("ALTER TABLE peers ADD COLUMN listen_port INTEGER DEFAULT 0")

            if "mtu" not in columns:
                conn.execute("ALTER TABLE peers ADD COLUMN mtu INTEGER DEFAULT 1280")

            if "server_id" not in columns:
                conn.execute("ALTER TABLE peers ADD COLUMN server_id INTEGER DEFAULT 1")

            conn.commit()

    # ========== 服务端管理 ==========

    def get_servers(self) -> list[ServerConfig]:
        """获取所有服务端"""
        with self._get_conn() as conn:
            rows = conn.execute("SELECT * FROM server ORDER BY id").fetchall()
            return [self._row_to_server(row) for row in rows]

    def get_server(self, server_id: int = 1) -> Optional[ServerConfig]:
        """获取指定服务端配置"""
        with self._get_conn() as conn:
            row = conn.execute("SELECT * FROM server WHERE id = ?", (server_id,)).fetchone()
            if row:
                return self._row_to_server(row)
        return None

    def get_server_by_endpoint(self, endpoint: str) -> Optional[ServerConfig]:
        """根据 endpoint 获取服务端"""
        with self._get_conn() as conn:
            row = conn.execute("SELECT * FROM server WHERE endpoint = ?", (endpoint,)).fetchone()
            if row:
                return self._row_to_server(row)
        return None

    def get_server_by_endpoint_and_interface(self, endpoint: str, interface: str) -> Optional[ServerConfig]:
        """根据 endpoint + interface 组合获取服务端"""
        with self._get_conn() as conn:
            row = conn.execute(
                "SELECT * FROM server WHERE endpoint = ? AND interface = ?",
                (endpoint, interface)
            ).fetchone()
            if row:
                return self._row_to_server(row)
        return None

    def get_servers_by_endpoint(self, endpoint: str) -> list[ServerConfig]:
        """根据 endpoint 获取所有服务端（同一服务器的多个接口）"""
        with self._get_conn() as conn:
            rows = conn.execute(
                "SELECT * FROM server WHERE endpoint = ? ORDER BY interface",
                (endpoint,)
            ).fetchall()
            return [self._row_to_server(row) for row in rows]

    def _row_to_server(self, row: sqlite3.Row) -> ServerConfig:
        """将数据库行转换为 ServerConfig 对象"""
        return ServerConfig(
            id=row["id"],
            private_key=row["private_key"],
            public_key=row["public_key"],
            address=row["address"],
            listen_port=row["listen_port"],
            interface=row["interface"],
            endpoint=row["endpoint"],
            post_up=row["post_up"],
            post_down=row["post_down"]
        )

    def save_server(self, server: ServerConfig) -> ServerConfig:
        """保存服务端配置（新增或更新）"""
        with self._get_conn() as conn:
            if server.id:
                # 更新现有服务端
                conn.execute("""
                    UPDATE server SET private_key = ?, public_key = ?, address = ?,
                        listen_port = ?, interface = ?, endpoint = ?, post_up = ?, post_down = ?
                    WHERE id = ?
                """, (server.private_key, server.public_key, server.address,
                      server.listen_port, server.interface, server.endpoint,
                      server.post_up, server.post_down, server.id))
            else:
                # 新增服务端
                cursor = conn.execute("""
                    INSERT INTO server (private_key, public_key, address, listen_port,
                                        interface, endpoint, post_up, post_down)
                    VALUES (?, ?, ?, ?, ?, ?, ?, ?)
                """, (server.private_key, server.public_key, server.address,
                      server.listen_port, server.interface, server.endpoint,
                      server.post_up, server.post_down))
                server.id = cursor.lastrowid
            conn.commit()
        return server

    def delete_server(self, server_id: int) -> bool:
        """删除服务端及其所有客户端"""
        with self._get_conn() as conn:
            conn.execute("DELETE FROM peers WHERE server_id = ?", (server_id,))
            cursor = conn.execute("DELETE FROM server WHERE id = ?", (server_id,))
            conn.commit()
            return cursor.rowcount > 0

    def get_ssh_config(self, server_id: int = 1) -> Optional[dict]:
        """获取 SSH 配置"""
        with self._get_conn() as conn:
            row = conn.execute(
                "SELECT ssh_host, ssh_port, ssh_user FROM server WHERE id = ?",
                (server_id,)
            ).fetchone()
            if row and row["ssh_host"]:
                return {
                    "host": row["ssh_host"],
                    "port": row["ssh_port"] or 22,
                    "user": row["ssh_user"] or "root"
                }
        return None

    def save_ssh_config(self, server_id: int, host: str, port: int = 22, user: str = "root"):
        """保存 SSH 配置"""
        with self._get_conn() as conn:
            conn.execute("""
                UPDATE server SET ssh_host = ?, ssh_port = ?, ssh_user = ?
                WHERE id = ?
            """, (host, port, user, server_id))
            conn.commit()

    # ========== 客户端管理 ==========

    def get_peers(self, server_id: int = 1) -> list[Peer]:
        """获取指定服务端的所有客户端"""
        with self._get_conn() as conn:
            rows = conn.execute(
                "SELECT * FROM peers WHERE server_id = ? ORDER BY created_at",
                (server_id,)
            ).fetchall()
            return [self._row_to_peer(row) for row in rows]

    def get_peer_by_name(self, name: str, server_id: int = 1) -> Optional[Peer]:
        """根据名称获取客户端"""
        with self._get_conn() as conn:
            row = conn.execute(
                "SELECT * FROM peers WHERE name = ? AND server_id = ?",
                (name, server_id)
            ).fetchone()
            if row:
                return self._row_to_peer(row)
        return None

    def _row_to_peer(self, row: sqlite3.Row) -> Peer:
        """将数据库行转换为 Peer 对象"""
        # 兼容旧数据库
        listen_port = row["listen_port"] if "listen_port" in row.keys() else 0
        mtu = row["mtu"] if "mtu" in row.keys() else 1280
        server_id = row["server_id"] if "server_id" in row.keys() else 1

        return Peer(
            id=row["id"],
            server_id=server_id,
            name=row["name"],
            public_key=row["public_key"],
            private_key=row["private_key"],
            preshared_key=row["preshared_key"],
            address=row["address"],
            allowed_ips=row["allowed_ips"],
            dns=row["dns"],
            listen_port=listen_port,
            mtu=mtu,
            created_at=row["created_at"],
            enabled=bool(row["enabled"])
        )

    def add_peer(self, peer: Peer) -> Peer:
        """添加客户端"""
        with self._get_conn() as conn:
            cursor = conn.execute("""
                INSERT INTO peers (server_id, name, public_key, private_key, preshared_key,
                                   address, allowed_ips, dns, listen_port, mtu,
                                   created_at, enabled)
                VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
            """, (peer.server_id, peer.name, peer.public_key, peer.private_key,
                  peer.preshared_key, peer.address, peer.allowed_ips, peer.dns,
                  peer.listen_port, peer.mtu, peer.created_at, 1 if peer.enabled else 0))
            conn.commit()
            peer.id = cursor.lastrowid
        return peer

    def remove_peer(self, name: str, server_id: int = 1) -> bool:
        """删除客户端"""
        with self._get_conn() as conn:
            cursor = conn.execute(
                "DELETE FROM peers WHERE name = ? AND server_id = ?",
                (name, server_id)
            )
            conn.commit()
            return cursor.rowcount > 0

    def toggle_peer(self, name: str, server_id: int = 1) -> Optional[bool]:
        """切换客户端启用状态"""
        with self._get_conn() as conn:
            row = conn.execute(
                "SELECT enabled FROM peers WHERE name = ? AND server_id = ?",
                (name, server_id)
            ).fetchone()
            if row:
                new_status = 0 if row["enabled"] else 1
                conn.execute(
                    "UPDATE peers SET enabled = ? WHERE name = ? AND server_id = ?",
                    (new_status, name, server_id)
                )
                conn.commit()
                return bool(new_status)
        return None

    def get_used_ips(self, server_id: int = 1) -> set[int]:
        """获取已使用的 IP 地址最后一段"""
        with self._get_conn() as conn:
            rows = conn.execute(
                "SELECT address FROM peers WHERE server_id = ?",
                (server_id,)
            ).fetchall()
            used = {1}  # 服务器使用 .1
            for row in rows:
                ip = row["address"].split("/")[0]
                parts = ip.split(".")
                if len(parts) == 4:
                    used.add(int(parts[-1]))
            return used
